@@ -35,6 +35,13 @@ class _Call(object):
                 and self.sample == other.sample
                 and self.gt_type == other.gt_type)
 
+    def __getstate__(self):
+        return dict((attr, getattr(self, attr)) for attr in self.__slots__)
+
+    def __setstate__(self, state):
+        for attr in self.__slots__:
+            setattr(self, attr, state.get(attr))
+
     def gt_phase_char(self):
         return "/" if not self.phased else "|"
 
@@ -133,7 +140,7 @@ class _Record(object):
         self.alleles = [self.REF]
         self.alleles.extend(self.ALT)
         #: list of ``_Calls`` for each sample ordered as in source VCF
-        self.samples = samples
+        self.samples = samples or []
         self._sample_indexes = sample_indexes
 
     def __eq__(self, other):
@@ -540,4 +547,8 @@ def make_calldata_tuple(fields):
                 for (x, y) in zip(self._fields, self)])
             return "CallData(" + dat + ')'
 
+        def __reduce__(self):
+            args = super(CallData, self).__reduce__()
+            return make_calldata_tuple, (fields, )
+
     return CallData
